<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 */

/**
 * This is a new version of one the Big5: the security class
 *
 * This new version supports connectors and queries class and should be compatible with the old one
 *
 * @version	1.0
 * @date 20100219
 * @author jason, greg, ross
 *
 */
class PHPDS_security extends security
{
	/**
	 * Cleaned up $_GET.
	 *
	 * @var mixed
	 */
	public $get;
	/**
	 * Cleaned up $_POST.
	 *
	 * @var mixed
	 */
	public $post;
	/**
	 * Cleaned up $_REQUEST.
	 *
	 * @var mixed
	 */
	public $request;
	/**
	 * Cleaned up $_SESSION.
	 *
	 * @var mixed
	 */
	public $session;
	/**
	 * Cleaned up $_COOKIE.
	 *
	 * @var mixed
	 */
	public $cookie;

	/**
	 * This method does the actual security check, other security checks are done on a per call basis to this method in specific scripts.
	 * Improved version reduces the cost of queries by 3, I also believe that this is a more secure method.
	 *
	 * @param boolean $validate_crypt_key Set if you would like the system to verify an encryption before accepting global $_POST variables. Use with method send_crypt_key_validation in your form.
	 * @return string
	 * @author Jason Schoeman
	 */
	public function securityIni($validate_token = false)
	{
		$this->_log(sprintf(___('Security check for user id %s'), $_SESSION['user_id']));

		// Do last security checks before allowing the script to execute.
		// Check if user can see this menu item.
		if (!empty($_GET['menu_id']) && !empty($this->navigation->navigation[$_GET['menu_id']]['menu_id'])) {
			$gm = $_GET['menu_id'];
			$sm = $this->navigation->navigation[$_GET['menu_id']]['menu_id'];
		} else {
			$gm = false;
			$sm = false;
		}
		// Show error if user does not have permission.
		if ($gm != $sm) {
			$this->template->stopScript = ___('SECURITY ALERT, There seems to be a security limitation, you are not allowed to run this script or the script could not be found. Make sure your session did not expire, you might be logged out.');
		}
		///////////////////////////////////////////////////////////////////////
		// IS SYSTEM ACTIVE OR SWITCHED OFF ///////////////////////////////////
		///////////////////////////////////////////////////////////////////////
		// ACTIVE - System is down.
		if ($this->configuration['system_down'] == true) {
			// ROOT - Allow only root users.
			if ($this->configuration['user_role'] == $this->configuration['root_role']) {
				if ($this->configuration['system_down_bypass'] == false) {
					$this->template->warning(___('System is switched off for normal users, only root can access the system.'), false, false);
				}
			} else if ($this->configuration['system_down_bypass'] == false) {
				$settings_message = $this->db->getSettings(array('system_down_message'));
				$this->template->skipLogin = true;
				$this->template->stopScript = sprintf($settings_message['system_down_message'], $this->configuration['scripts_name_version']);
			}
		}
		///////////////////////////////////////////////////////////////////////
		///////////////////////////////////////////////////////////////////////
		///////////////////////////////////////////////////////////////////////
		// Prevent SQL injections.
		// Magic quotes should not be used anymore.
		// For now only handle these by default.
		// Trim if normal.
		if (!empty($_POST)) $this->post = $this->sqlWatchdog($_POST);
		if (!empty($_GET)) $this->get = $this->sqlWatchdog($_GET);
		if (!empty($_COOKIE)) $this->cookie = $this->sqlWatchdog($_COOKIE);
		if (!empty($_REQUEST)) $this->request = $this->sqlWatchdog($_REQUEST);
		if (!empty($_SESSION)) $this->session = $this->sqlWatchdog($_SESSION);
		// Check if $this->post came from an external page, it needs to be validated through a token system.
		if ($validate_token == true && !empty($this->post)) {
			// Create md5 key multi session array with crypt key.
			$key = md5($this->configuration['crypt_key']);
			// ->>>>>>>>>>>>>>
			// Set.
			if (!empty($_SESSION['token_validation'][$key])) {
				$token_validation_key = $_SESSION['token_validation'][$key];
			} else {
				$token_validation_key = 1;
			}
			// Set.
			if (!empty($this->post['token_validation'])) {
				$token_validation__ = $this->post['token_validation'];
			} else {
				$token_validation__ = 2;
			}
			// Check if script knew token, this way we will know if this script is forged.
			if ($token_validation_key !== $token_validation__) {
				// Create crypt key multi session array with crypt key.
				$search_key = md5(sha1($this->configuration['crypt_key']));
				// This might have been the search filter that requested the $this->post.
				if ((empty($this->post['search_token_validation'])) || ($_SESSION['token_validation'][$search_key] !== $this->post['search_token_validation'])) {
					// Now all else have failed, this must be a hacking attempt.
					// If the form did not know the token, give an alert!
					$this->template->stopScript = ___('SECURITY ALERT, a Security check was enforced, please try again!');
				}
			}
			// Lets just destroy the session token to clean things up.
			unset($_SESSION['token_validation']);
		}
	}

	/**
	 * Function just like mysql_real_escape_string, but does so recursive through array.
	 *
	 * @param mixed $input
	 */
	public function sqlWatchdog($input)
	{
		if (is_array($input)) {
			foreach ($input as $k => $i) {
				$output[$k] = $this->sqlWatchdog($i);
			}
		} else {
			// Convert characters to their html counterparts.
			$output = trim(htmlentities(str_replace('\\', '', $input), ENT_QUOTES, $this->configuration['charset']));
		}
		return $output;
	}

	/**
	 * Use inside your form brackets to send through a token validation to limit $this->post received from external pages.
	 *
	 * @return string Returns hidden input field.
	 */
	public function postValidation()
	{
		return $this->validatePost();
	}

	/**
	 * Use inside your form brackets to send through a token validation to limit $this->post received from external pages.
	 *
	 * @return string Returns hidden input field.
	 */
	public function validatePost()
	{
		// Lets create a random validation token.
		$token = md5(uniqid(rand(), TRUE));
		// Lets make it even harder for the naughty and create a multi session array with a MD5 crypt key.
		$key = md5($this->configuration['crypt_key']);
		// ->>>>>>>>>>>>>>
		// Assign a session token.
		// Now the hacker will need to know the multidimentional session before he can attempt any hacking.
		$_SESSION['token_validation'][$key] = $token;
		// Return form.
		return '<input type="hidden" name="token_validation" value="' . $token . '">';
	}

	/**
	 * This is used in the search filter to validate $this->post made by the search form.
	 *
	 * @return string Returns hidden input field.
	 */
	public function searchFormValidation()
	{
		// Lets create a random validation token.
		$search_token = md5(uniqid(rand(), TRUE));
		// Lets make it even harder for the naughty and create a multi session array with a crypt crypt key.
		$search_key = md5(sha1($this->configuration['crypt_key']));
		// ->>>>>>>>>>>>>>
		// Assign a session token.
		// Now the hacker will need to know the multidimentional session before he can attempt any hacking.
		$_SESSION['token_validation'][$search_key] = $search_token;
		// Return form.
		return '<input type="hidden" name="search_token_validation" value="' . $search_token . '">';
	}

	/**
	 * Search the database for the given credentials
	 *
	 * @deprecated
	 * @param string $username
	 * @param string $password
	 * @return array or false the user record
	 * @date 20100204 (v1.0)(greg) created from Jason's original fct
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @version	1.0.1
	 * @author jason, greg
	 */
	public function lookupUser($username, $password = '')
	{
		return $this->user->lookupUser($username, $password);
	}

	/**
	 * Search the database for the given credentials from a persistent cookie
	 *
	 * @deprecated
	 * @param string $cookie
	 * @return array or false the user record
	 * @date 20100702 (v1.0.0)(ross) created from the lookup_user fct by jason and greg
	 * @version	1.0.0
	 * @author jason, greg, ross
	 */
	public function lookupPersistentUser($cookie)
	{
		return $this->user->lookupPersistentUser($cookie);
	}

	/**
	 * Set a persistent cookie to be used as a remember me function
	 *
	 *
	 * @param string $cookie
	 * @return array or false the user record
	 * @date 20100702 (v1.0.0) (ross) created function
	 * @version	1.0.0
	 * @author ross
	 */
	public function setPersistentCookie($user_id)
	{
		return $this->user->setPersistentCookie($user_id);
	}

	/**
	 * Delete the current persistent cookie from the db and kill the cookie on the user end.
	 *
	 * @deprecated
	 * @return boolean
	 * @date 20100702 (v1.0.0) (ross) created function
	 * @version	1.0.0
	 * @author ross
	 */
	public function deletePersistentCookie($user_id)
	{
		return $this->user->deletePersistentCookie($user_id);
	}

	/**
	 * Sets all settings to guest account.
	 *
	 * @deprecated
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @return string
	 * @author Jason Schoeman, greg
	 */
	public function setGuest()
	{
		$this->user->setGuest();
	}

	/**
	 * Check if user is a root user.
	 *
	 * @deprecated
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @param mixed $user_id If not logged in user, what user should be checked (primary role check only).
	 * @return boolean
	 */
	public function isRoot($user_id = false)
	{
		return $this->user->isRoot($user_id);
	}

	/**
	 * Returns current logged in user id.
	 *
	 * @deprecated
	 * @return integer
	 */
	public function currentUserID()
	{
		return $this->configuration['user_id'];
	}

	/**
	 * Check if the currently logged in user is the same group as the given user
	 *
	 * This can be used to check if the current user is allowed access to the given user's data
	 *
	 * @deprecated
	 * @date 20100222
	 * @version	1.0
	 * @author greg
	 * @param $user_id integer, the ID of the other user
	 * @return boolean, whether the current user is in the same group
	 * @see	http://wiki.phpdevshell.org/wiki/Security_with_groups_and_roles
	 */
	public function isSameGroup($user_id)
	{
		return $this->user->isSameGroup($user_id);
	}

	/**
	 * Destroys login session data.
	 * @deprecated
	 * @author Jason Schoeman
	 */
	public function logout($set_guest = true)
	{
		$this->user->logout($set_guest);
	}

	/**
	 * Encrypts a string with the configuration key provided.
	 *
	 * @param string $string
	 * @return string
	 */
	public function encrypt($string)
	{
		$result = false;
		for ($i = 0; $i < strlen($string); $i++) {
			$char = substr($string, $i, 1);
			$keychar = substr($this->configuration['crypt_key'], ($i % strlen($this->configuration['crypt_key'])) - 1, 1);
			$char = chr(ord($char) + ord($keychar));
			$result .= $char;
		}
		return urlencode(base64_encode($result));
	}

	/**
	 * Decrypts a string with the configuration key provided.
	 *
	 * @param string $string
	 * @return string
	 */
	public function decrypt($string)
	{
		$result = false;
		$string = base64_decode(urldecode($string));
		for ($i = 0; $i < strlen($string); $i++) {
			$char = substr($string, $i, 1);
			$keychar = substr($this->configuration['crypt_key'], ($i % strlen($this->configuration['crypt_key'])) - 1, 1);
			$char = chr(ord($char) - ord($keychar));
			$result .= $char;
		}
		return $result;
	}

	/**
	 * Checks to see if user and password is correct and allowed. Then creates session data accordingly.
	 *
	 * @param string $username
	 * @param string $password
	 * @return boolean
	 * @date 20100204 greg: split into pieces
	 * @version	1.1
	 * @author jason, greg
	 */
	public function login($username, $password)
	{
		return $this->user->login($username, $password);
	}

	/**
	 * Check is user is logged in, return false if not.
	 *
	 * @deprecated
	 * @return boolean
	 */
	public function isLoggedIn()
	{
		return $this->user->isLoggedIn();
	}

	/**
	 * Check if a user has access to a given menu id.
	 *
	 * @version 1.0.1
	 * @date 20091105 fixed a possible warning when the menu is not in the list (i.e. the user is not allowed)
	 * @deprecated
	 * @param mixed This can have both the menu id as an integer or as a string.
	 * @param string The type of item requested, menu_id, menu_name etc...
	 * @return boolean Will return requested variable if user has access to requested menu item menu item.
	 */
	public function canAccessMenu($menu_id, $type = 'menu_id')
	{
		return $this->user->canAccessMenu($menu_id, $type);
	}

	/**
	 * Make the given user the logged in user
	 *
	 * @deprecated
	 * @param array $select_user_array
	 * @return nothing
	 * @date 20100204 greg: created from Jason's original fct
	 * @version	1.0
	 * @author jason, greg
	 */
	public function doLogin($select_user_array, $persistent = false)
	{
		$this->user->doLogin($select_user_array, $persistent);
	}

	/**
	 * Creates a "secret" version of the password
	 *
	 * @param string $password, the clear password
	 * @return string the hashed password
	 * @date 20100204 greg: created from Jason's original fct
	 * @version	1.0
	 * @author jason, greg
	 */
	public function hashPassword($password = '')
	{
		return empty($password) ? '*' : md5($password);
	}

	/**
	 * Simple method to return users IP, this method will be improved in the future if needed.
	 *
	 * @deprecated
	 * @return string
	 */
	public function userIp()
	{
		return $this->user->getUserIp();
	}

	/**
	 * Simple method to return users IP, this method will be improved in the future if needed.
	 *
	 * @deprecated
	 * @return string
	 */
	public function getUserIp()
	{
		return $this->user->getUserIp();
	}

	/**
	 * Validates email address.
	 *
	 * @param string Email address.
	 * @return boolean
	 * @author Jason Schoeman
	 */
	public function validateEmail($email_string)
	{
		if (filter_var($email_string, FILTER_VALIDATE_EMAIL) == TRUE) {
			return true;
		} else {
			return false;
		}
	}
}