<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Control Panel Model - Menu Items.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class PHPDS_faviMenusQuery extends PHPDS_query
{
	protected $sql = "
		SELECT COUNT(t1.log_id) as hits,
			t1.menu_id, t1.user_id,
			t2.hide
		FROM
			_db_core_menu_access_logs t1
		LEFT JOIN
			_db_core_menu_items t2
		ON
			t1.menu_id = t2.menu_id
		WHERE
			t1.user_id = '%s'
		AND
			t1.menu_id != 940041356
		AND
			t2.hide IN (0, 2, 3)
		AND
			t2.menu_type IN (1, 4, 5, 7)
		GROUP BY
			menu_id
		ORDER BY
			hits
		DESC LIMIT %d
		";

	/**
	 * Initiate query invoke command.
	 * @param int
	 * @return array
	 */
	public function invoke ($parameters)
	{
		// create the parameter list for the query
		// we add an argument there, otherwise we could just pass the parameters to the query() method
		$limit_favorite_db = intval($parameters[0]);
		$parameters = array($this->configuration['user_id'], $limit_favorite_db);

		// send the actual query to the database
		$favi_menus = parent::invoke($parameters);

		// ok now the real job, the data is here, no DB-related stuff anymore
		$favi_menu_item = array();
		if (! empty($favi_menus)) {
			foreach ($favi_menus as $favi_menus_array) {
				$favi_menu_item[$favi_menus_array['menu_id']] = $favi_menus_array['menu_id'];
			}
		}
		if (! empty($favi_menu_item)) {
			return $favi_menu_item;
		} else {
			return false;
		}
	}

}
/**
 * Initiate query invoke command.
 * @param int
 * @return array
 */
class PHPDS_logsQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			t1.id, t1.log_type, t1.log_description, t1.log_time
		FROM
			_db_core_logs t1
		WHERE
			t1.user_id = %u
		AND
			t1.user_id != 0
		AND
			t1.log_type IN (1,2,3)
		ORDER BY
			t1.log_time
		DESC
		LIMIT 0, %u
	";

	/**
	 * Loads array of log messages.
	 * @param int $limit_messages
	 * @return array
	 */
	public function invoke ($parameters)
	{
		// create the parameter list for the query
		// we add an argument there, otherwise we could just pass the parameters to the query() method
		$limit_messages = $parameters[0];
		$parameters = array($this->configuration['user_id'], $limit_messages);
		$template = $this->template;
		// send the actual query to the database
		$log_enties = parent::invoke($parameters);

		if (!empty($log_enties)) {
			// ok now the real job, the data is here, no DB-related stuff anymore
			$message_ = array();
			foreach($log_enties as $get_logs_array) {
				$log_type = $get_logs_array['log_type'];
				$log_description = $get_logs_array['log_description'];
				$log_time = $this->core->formatTimeDate($get_logs_array['log_time']);
				// Set messages in correct categories.
				switch ($log_type) {
					// Ok.
					case 1:
						$message_[] = array('description'=>sprintf($template->module['ok'], $log_description), 'log_time'=>$log_time);
						break;
					// Warning.
					case 2:
						$message_[] = array('description'=>sprintf($template->module['warning'], $log_description), 'log_time'=>$log_time);
						break;
					// Critical.
					case 3:
						$message_[] = array('description'=>sprintf($template->module['critical'], $log_description), 'log_time'=>$log_time);
						break;
				}
			}
		}
		if (! empty($message_)) {
			return $message_;
		} else {
			return false;
		}
	}
}
/**
 * Control Panel Model - Draw Control Panel
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class PHPDS_drawCPModel extends PHPDS_query
{
	/**
	 * Initiate query invoke command.
	 * @param int
	 * @return array
	 */
	public function invoke ($parameters)
	{
		$favi = $parameters[0];
		$behaviour = $parameters[1];
		
		if ($behaviour == 'restricted') {
			$restict = true;
		} else {
			$restict = false;
		}
		$navigation = $this->navigation;
		$configuration = $this->configuration;
		$template = $this->template;
		// Assign parent menu item to let control panel know when to stop processing links.
		$parent_stop_id = $navigation->navigation[$this->configuration['m']]['parent_menu_id'];
		// Set.
		$start_cp = false;
		// Loop through all menu items.
		foreach ($navigation->navigation as $select_menu_items_array) {
			// Assign needed items for check.
			$item['menu_id'] = $select_menu_items_array['menu_id'];
			// Only start showing when active menu item is reached.
			if (($item['menu_id'] == $configuration['m']) || ($start_cp == true) || ($navigation->navigation[$configuration['m']]['parent_menu_id'] == 0 && $navigation->navigation[$configuration['m']]['is_parent'] == 0)) {
				// Start control panel.
				$start_cp = true;
				// Assign rest of menu items.
				$item['parent_menu_id'] = $select_menu_items_array['parent_menu_id'];
				$item['menu_name'] = $select_menu_items_array['menu_name'];
				$item['plugin'] = $select_menu_items_array['plugin'];
				$item['menu_link'] = $select_menu_items_array['menu_link'];
				$item['alias'] = $select_menu_items_array['alias'];
				$item['is_parent'] = $select_menu_items_array['is_parent'];
				$item['hide'] = $select_menu_items_array['hide'];
				$item['type'] = $select_menu_items_array['type'];
				$item['menu_type'] = $select_menu_items_array['menu_type'];
				$item['new_window'] = $select_menu_items_array['new_window'];
				// Should we show this item.
				if (($item['hide'] == 0 || $item['hide'] == 3) && (($item['menu_type'] == 1) || ($item['menu_type'] == 5))) {
					// Check exit loop.
					$active_parent_menu_id = $item['parent_menu_id'];
					// Halt control panel link creation when cycle is reached.
					if ($restict == true) {
						if (($active_parent_menu_id == $parent_stop_id) && ($configuration['m'] != 940041356)) break;
					}
					// Get script image/logo.
					$image_url = $template->scriptLogo($item['menu_link'], $item['plugin'], $item['alias']);

					// Open in new window.
					if ($item['new_window'] == 1) {
						$newWindow = 'target="_blank"';
					} else {
						$newWindow = false;
					}

					// Should we highlight
					if (! empty($favi[$item['menu_id']])) {
						$class = 'favi-active';
					} else {
						$class = 'favi-inactive';
					}
					// Set url.
					$url = $navigation->buildURL($item['menu_id'], 'l');
					// Check if menu item is a parent and assign correct div class.
					$menu_type[] = array('class'=>$class, 'url'=>$url, 'newWindow'=>$newWindow, 'menu_name'=>$item['menu_name'], 'image_url'=>$image_url );
				}
			}
		}
		if (! empty($menu_type)) {
			return $menu_type;
		} else {
			return array();
		}
	}
}