<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: User Administration
 * @author Jason Schoeman
 * @return string
 */
class UserAdmin extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		$sa = $this->db->getSettings();

		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Load Extra Classes ////////////////////////////////////////////////////////////////////////////////////////////////////
		$email = $this->factory('mailer'); ///////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Require JS.
		$this->template->addJsFileToHead('themes/cloud/js/multiselect/js/plugins/scrollTo/jquery.scrollTo-min.js');
		$this->template->addJsFileToHead('themes/cloud/js/multiselect/js/ui.multiselect.js');
		$this->template->addJsFileToHead('themes/cloud/js/multiselect/jquery.multiselect.js');
		// Define.
		$user_role_option = '';
		$edit_ = array('user_id' => '', 'user_role' => '', 'user_group' => '', 'language' => '', 'region' => '', 'user_timezone' => '', 'user_name' => '', 'user_display_name' => '', 'user_email' => '', 'date_registered' => '');
		$edit = $edit_;
		$extra_roles_role_option = '';
		$user_role_id_form = array();
		$extra_groups_group_option = '';
		$user_group_id_form = array();
		$user_group_option = '';
		$role_id_db = 0;
		// Header.
		if (!empty($this->security->get['eu']) || !empty($this->security->post['save'])) {
			$this->template->heading(_('Edit User'));
			$this->template->info(_('Edit existing users on the system.'));
		} else {
			$this->template->heading(_('Add Users'));
			$this->template->info(_('Used to add new users to the system. These users will have the ability to log in and access menus and menu roles where permissions apply.'));
			$this->template->notice(_('Leave the PASSWORD field *blank* to have password auto generated and emailed to the user!'));
		}
		// Check if it is a user that is been updated.
		if (!empty($this->security->get['eu']) || !empty($this->security->post['save']) || !empty($this->security->post['new'])) {
			// If Edit
			if (!empty($this->security->get['eu'])) {
				// Password "leave blank" notice.
				$this->template->notice(_('Leave the PASSWORD field *blank* to have password unchanged!'));
				$edit = $this->db->invokeQuery('PHPDS_readUserQuery', $this->security->get['eu']);
				// Check if we have a prankster, who tries to edit but does not belong to him.
				if (empty($edit['user_id'])) {
					$this->template->warning(_('You cannot edit a user that does not belong to you!?'));
					$edit = $edit_;
				}
			}
			// If Save...
			if (!empty($this->security->post['save']) || !empty($this->security->post['new'])) {
				$edit = $this->security->post;
				if (empty($edit['user_id'])) $edit['user_id'] = '';
				$edit['user_name'] = trim($this->security->post['user_name']);
				// Define.
				if (empty($edit['language'])) {
					$edit['language'] = '';
					$edit['region'] = '';
				}

				if (empty($edit['user_name']) || empty($edit['user_display_name']) || empty($edit['user_email'])) {
					$this->template->warning(_('You have missed some fields, try again!'));
				} else {
					// Do the matching query to check if any user exists.
					$check_user_array = $this->db->invokeQuery('PHPDS_readUserDetailQuery', $edit['user_id'], $edit['user_name'], $edit['user_email']);
					// Get the results.
					$user_name_ = trim($check_user_array['user_name']);
					$user_display_name_ = trim($check_user_array['user_display_name']);
					$user_email_ = trim($check_user_array['user_email']);

					if ($user_name_ == $edit['user_name']) {
						$this->template->warning(_('The username you entered already exists, please specify a different username.'));
						$error[1] = true;
					}
					if ($user_email_ == $edit['user_email']) {
						$this->template->warning(_('The user email you entered already exists, is this user perhaps already registered?'));
						$error[3] = true;
					}
					// Check if these roles and groups are allowed to be saved.
					if (!$this->user->belongsToGroup(false, $edit['user_group']) && !$this->user->belongsToRole(false, $edit['user_role'])) {
						$this->template->warning(_('You cannot edit a user that does not belong to you!?'));
						$error[4] = true;
					}
					if (empty($edit['user_group'])) {
						$this->template->warning(_('You cannot leave the primary group empty, you wont have access to this user anymore!?'));
						$error[5] = true;
					}

					// Check to see if all fields are set.
					if (empty($error)) {
						// Check if we need to create a new user or not...
						if (empty($edit['user_id'])) {
							// Check if password should be kept the same.
							if (!empty($edit['user_password'])) {
								$insert_encrypt_pass = md5($edit['user_password']);
								$send_user_pass = $edit['user_password'];
							} else {
								$password_generate = $this->core->createRandomString(10, false);
								$insert_encrypt_pass = md5($password_generate);
								$send_user_pass = $password_generate;
							}
							// Get last insert user_id.
							$edit['user_id'] = $this->db->invokeQuery('PHPDS_writeUserQuery', $edit['user_display_name'], $edit['user_name'], $insert_encrypt_pass, $edit['user_email'], $edit['user_group'], $edit['user_role'], $edit['date_registered'], $edit['language'], $edit['user_timezone'], $edit['region']);
							// Set variable global for hooks.
							$this->template->global['user_id'] = $edit['user_id'];

							// Send notification email.
							if (!empty($edit['send_notification']))
									$email->sendmail("{$edit['user_email']}", sprintf(_('Login profile created at %s'), $this->configuration['absolute_url']), sprintf(_("%s,\r\na Profile was created at %s,\r\n\r\nYour username : %s\r\nYour password : %s\r\nFor security reasons, please change your password when logging in!"), $edit['user_display_name'], $this->configuration['scripts_name_version'], $edit['user_name'], $send_user_pass));
						} else {
							// Check if password should be kept the same.
							if (!empty($edit['user_password'])) {
								$send_user_pass = $edit['user_password'];
							} else {
								$this->template->notice(_('User password was left unchanged.'));
								$send_user_pass = _('Unchanged');
							}
							// Update core detail.
							$this->db->invokeQuery('PHPDS_updateUserQuery',
									$edit['user_display_name'],
									$edit['user_name'],
									$edit['user_password'],
									$edit['user_email'],
									$edit['date_registered'],
									$edit['user_group'],
									$edit['user_role'],
									$edit['language'],
									$edit['user_timezone'],
									$edit['region'],
									$edit['user_id']
							);
							// Set variable global for hooks.
							$this->template->global['user_id'] = $edit['user_id'];

							// Send notification email.
							if (!empty($edit['send_notification']))
									$email->sendmail("$edit[user_email]", sprintf(_('Login profile updated at %s'), $this->configuration['absolute_url']), sprintf(_("%s,\r\nYour profile was updated at %s,\r\n\r\nYour username : %s\r\nYour password : %s\r\n\r\nChange your password if password was received by mail!"), $edit['user_display_name'], $this->configuration['scripts_name_version'], $edit['user_name'], $send_user_pass));
						}
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						//////////////////////////////////////////////////////// Extra Roles ////////////////////////////////////////////////////////
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						// Delete old extra roles values.
						$this->db->invokeQuery('PHPDS_deleteExtraRolesQuery', $edit['user_id']);
						// Check if any extra roles items exists.
						if (!empty($edit['extra_roles'])) {
							// Update extra roles detail.
							$this->db->invokeQuery('PHPDS_replaceExtraRolesQuery', $edit['extra_roles'], $edit['user_id']);
						} else {
							unset($edit['extra_roles']);
						}
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						///////////////////////////////////////////////////// END Extra Roles ///////////////////////////////////////////////////////
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						//////////////////////////////////////////////////////// User Groups ////////////////////////////////////////////////////////
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						// Delete old groups values.
						$this->db->invokeQuery('PHPDS_deleteExtraGroupsQuery', $edit['user_id']);
						// Check if any user groups items exists.
						if (!empty($edit['extra_groups'])) {
							// Update user group detail.
							$this->db->invokeQuery('PHPDS_replaceExtraGroupsQuery', $edit['extra_groups'], $edit['user_id']);
						} else {
							unset($edit['extra_groups']);
						}
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						///////////////////////////////////////////////////// END extra_roles ///////////////////////////////////////////////////////
						/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
						$this->template->ok(sprintf(_('User %s saved.'), $edit['user_display_name']));
					}
				}
			}
			///////////////////////////////////
			// Select extra roles data.
			$user_role_id_form = $this->db->invokeQuery('PHPDS_selectedRolesQuery', $edit['user_id']);
			///////////////////////////////////
			///////////////////////////////////
			// Select user group data.
			$user_group_id_form = $this->db->invokeQuery('PHPDS_selectedGroupsQuery', $edit['user_id']);
			///////////////////////////////////
			// Save and new item.
			if (!empty($this->security->post['new'])) {
				$edit['user_id'] = 0;
				$edit['user_name'] = '';
				$edit['user_display_name'] = '';
				$edit['user_email'] = '';
				$edit['user_password'] = '';
			}
		}
		//////////////////////////
		// Get all user roles.  //
		//////////////////////////
		// Loop and see what needs to be selected.
		$roles = $this->db->invokeQuery('PHPDS_readRolesQuery');
		if (empty($edit['user_role'])) $edit['user_role'] = $sa['move_verified_role'];
		$user_role_option = PU_buildHTMLoptions($roles, $edit['user_role']);
		$extra_roles_role_option = PU_buildHTMLoptions($roles, array_keys($user_role_id_form));

		//////////////////////////
		// Get all user groups. //
		//////////////////////////
		// Loop user groups and see what needs to be selected.
		$groups = $this->db->invokeQuery('PHPDS_PrimaryGroupTreeQuery');
		if (empty($edit['user_group']))
			$edit['user_group'] = $sa['move_verified_group'];
		$user_group_option = PU_buildHTMLoptions($groups, $edit['user_group']);
		$extra_groups_group_option = PU_buildHTMLoptions($groups, array_keys($user_group_id_form));

		//////////////////////////
		//////////////////////////
		//////////////////////////
		// Set IANA language repository class.
		$iana = $this->factory('iana');
		$language_options = $iana->languageOptions($edit['language']);
		$region_options = $iana->regionOptions($edit['region']);
		// Get TimeZone options.
		$timezone = $this->factory('timeZone');
		$timezone_options = $timezone->timezoneOptions($edit['user_timezone']);

		// Load views.
		$view = $this->factory('views');

		// Set Array.
		$view->set('e', $edit);

		// Set Values.
		$view->set('self_url', $this->navigation->selfUrl());
		$view->set('user_role_option', $user_role_option);
		$view->set('extra_roles_role_option', $extra_roles_role_option);
		$view->set('user_group_option', $user_group_option);
		$view->set('extra_groups_group_option', $extra_groups_group_option);
		$view->set('language_options', $language_options);
		$view->set('region_options', $region_options);
		$view->set('locale', $this->configuration['locale']);
		$view->set('timezone_options', $timezone_options);
		$view->set('date_format_show', $this->core->formatTimeDate($this->configuration['time'], 'default', $edit['user_timezone']));
		$view->set('post_validation', $this->security->postValidation());
		// Output Template.
		$view->show();
	}
}

return 'UserAdmin';
