<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: Edit user preferences.
 * @author Jason Schoeman
 * @return string
 */
class EditPreferences extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		// Initiate Email Class.
		$email = $this->factory('mailer');
		$this->template->heading(_('Edit Accounts Preferences'));
		$this->template->info(_('This script is used to edit your existing preferences.'));
		$this->template->notice(_('Please be aware that changing these settings and forgetting it will cause you to be locked out of the system.'));

		$user_id = $this->configuration['user_id'];
		if ($this->user->isLoggedIn()) {
			$user_array = $this->db->invokeQuery('PHPDS_readUserDetailQuery', $user_id);
			$edit = $user_array;
			if (empty($edit['user_pass'])) $edit['user_pass'] = '';
		}
		if (!empty($this->security->post['save'])) {
			$edit['user_name'] = $this->security->post['user_name'];
			$edit['user_display_name'] = $this->security->post['user_display_name'];
			$edit['user_email'] = $this->security->post['user_email'];
			(!empty($this->security->post['language'])) ? $edit['language'] = $this->security->post['language'] : $edit['language'] = false;
			(!empty($this->security->post['region'])) ? $edit['region'] = $this->security->post['region'] : $edit['region'] = false;
			(!empty($this->security->post['user_timezone'])) ? $edit['user_timezone'] = $this->security->post['user_timezone'] : $edit['user_timezone'] = $this->configuration['system_timezone'];

			// If all fields are completed go to next level.
			if (empty($edit['user_name']) || empty($edit['user_display_name']) || empty($edit['user_email'])) {
				$this->template->warning(_('You have missed some fields, try again!'), false, false);
			} else {
				// Do the matching query to check if any user exists.
				$check_user_array = $this->db->invokeQuery('PHPDS_readUserDetailLightQuery', $user_id, $edit['user_name'], $edit['user_display_name'], $edit['user_email']);
				// Get the results.
				$user_name_ = $check_user_array['user_name'];
				$user_display_name_ = $check_user_array['user_display_name'];
				$user_email_ = $check_user_array['user_email'];
				if ($user_name_ == $edit['user_name']) {
					$this->template->warning(_('The username you entered already exists, please specify a different username.'), false, false);
					$field1 = 'boxerror';
					$error[1] = true;
				}
				if ($user_display_name_ == $edit['user_display_name']) {
					$this->template->warning(_('The user display name you entered already exists, please specify a different display name.'), false, false);
					$field2 = 'boxerror';
					$error[2] = true;
				}
				if ($user_email_ == $edit['user_email']) {
					$this->template->warning(_('The user email you entered already exists, are you perhaps already registered? If not please specify a different user email address.'), false, false);
					$field3 = 'boxerror';
					$error[3] = true;
				}
				// Check email string validity.
				if (!$email->validate($edit['user_email'])) {
					$this->template->warning(_('The email address you specified seems to be invalid. Please verify that your email address is entered correctly.'), false, false);
					$field3 = 'boxerror';
					$error[4] = true;
				}

				// Check to see if all fields are set.
				if (empty($error)) {
					$this->db->invokeQuery('PHPDS_updateUserDetailQuery', $edit['user_display_name'], $edit['user_name'], $edit['user_email'], $edit['language'], $edit['user_timezone'], $edit['region'], $user_id);
					// Set variable global for hooks.
					$this->template->global['user_id'] = $user_id;

					// Send notification email.
					$email->sendmail("$edit[user_email]", sprintf(_('User profile modified at %s.'), $this->configuration['absolute_url']), sprintf(_("%s you have modified your user settings at %s.\r\n\r\nThank You,\r\n%s"), $edit['user_display_name'], $this->configuration['scripts_name_version'], $this->configuration['absolute_url']));
					// Ok, lets tell user that all seems ok...
					$this->template->ok(_('All settings updated, please use new settings the next time you log in.'));
					$this->navigation->redirect(false, 3);
				}
			}
		}
		// Set IANA language repository class.
		$iana = $this->factory('iana');
		$language_options = $iana->languageOptions($edit['language']);
		$region_options = $iana->regionOptions($edit['region']);
		// Get TimeZone options.
		$timezone = $this->factory('timeZone');
		$timezone_options = $timezone->timezoneOptions($edit['user_timezone']);
		// Check to see if field values are assigned.
		if (empty($field1)) $field1 = 'boxdisabled';
		if (empty($field2)) $field2 = 'boxmand';
		if (empty($field3)) $field3 = 'boxmand';
		if (empty($field4)) $field4 = 'boxmand';

		// Load views.
		$view = $this->factory('views');

		$view->set('e', $edit);

		// Set Values.
		$view->set('self_url', $this->navigation->selfUrl());
		$view->set('field1', $field1);
		$view->set('field4', $field4);
		$view->set('field2', $field2);
		$view->set('field3', $field3);
		$view->set('language_options', $language_options);
		$view->set('region_options', $region_options);
		$view->set('timezone_options', $timezone_options);
		$view->set('date_format_show', $this->core->formatTimeDate($this->configuration['time'], 'default', $edit['user_timezone']));
		$view->set('post_validation', $this->security->validatePost());

		// Output Template.
		$view->show();
	}
}

return 'EditPreferences';
