<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: Handles registration finalization.
 * @author Jason Schoeman
 * @return string
 */
class NewPassword extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Load Extra Classes ////////////////////////////////////////////////////////////////////////////////////////////////////
		$email = $this->factory('mailer'); ///////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Require JS.
		$this->template->addJsFileToHead('themes/cloud/js/password/jquery.password.js');
		$shortPass = _('Too Short');
		$badPass = _('Weak');
		$goodPass = _('Good');
		$strongPass = _('Strong');
		$samePass = _('Username and Password identical!');
		$passwordMeter = <<<JS
           $(document).ready(function() {
                    $.fn.shortPass = '{$shortPass}';
                    $.fn.badPass = '{$badPass}';
                    $.fn.goodPass = '{$goodPass}';
                    $.fn.strongPass = '{$strongPass}';
                    $.fn.samePassword = '{$samePass}';
                    $.fn.resultStyle = "";
                    $(".password_test").passStrength({
                        shortPass: 		"critical",
                        badPass:		"warning",
                        goodPass:		"notice",
                        strongPass:		"ok",
                        baseStyle:		"passwordMeter",
                        userid:         "#user_name_test"
                    });
            });
JS;
		$this->template->addJsToHead($passwordMeter);
		// Set.
		$password1 = false;
		$password2 = false;
		$this->template->heading(_('Create new Password'));
		$this->template->info(_('Please choose a new password you would like to use to access this system.'));
		// Check if we have the required information.
		if (! empty($this->configuration['user_id'])) {
			$username = $this->configuration['user_name'];
			$eun = '';
		} else if (! empty($this->security->request['eun'])) {
			$username = $this->security->decrypt($this->security->request['eun']);
			$eun = $this->security->request['eun'];
		} else {
			$username = '';
			$eun = '';
		}

		if (empty($username)) {
			$this->template->warning('You cannot change your password directly please use the password recovery page.');
		} else {
			// Get user data.
			$edit = $this->db->invokeQuery('PHPDS_ReadUserQuery', $username);
			if (empty($edit['user_name'])) {
				$this->template->warning(_('This user cannot be found! Please try resetting your password again.'));
			} else {
				// Sending password instructions.
				if (isset($this->security->post['replace'])) {
					// Check for missing fields.
					if (empty($this->security->post['password1']) || empty($this->security->post['password2'])) {
						$this->template->warning(_('You need to enter both password fields!'), false, false);
						$error[0] = true;
					} else {
						// Does password match.
						if ($this->security->post['password1'] !== $this->security->post['password2']) {
							$this->template->warning(_('Your passwords does not match, please try again!'), false, false);
							$error[1] = true;
						}
						// Check if password is correct length.
						if (strlen($this->security->post['password1']) < 6) {
							$this->template->warning(_('Your password is too short, make it at least 6 characters!'), false, false);
							$field4 = 'boxerror';
							$error[2] = true;
						}
					}

					if (empty($error)) {
						if ($edit['user_name'] == $username) {
							// MD5 Password.
							$md5_password = md5($this->security->post['password1']);
							// Update the database
							$this->db->invokeQuery('PHPDS_UpdateUserQuery', $md5_password, $edit['user_name'], $edit['user_email']);
							$to = $edit['user_email'];
							$subject = sprintf(_('%s password changed.'), $this->configuration['scripts_name_version']);
							$message = sprintf(_("Dear %s member,\r\n\r\nYou have changed your password successfully.\r\n\r\nThank you,\r\n%s"), $this->configuration['scripts_name_version'], $this->configuration['absolute_url']);
							// Send new password email.
							if ($email->sendmail("$to", $subject, $message)) {
								$this->template->ok(sprintf(_('You have changed your password successfully %s'), $edit['user_display_name']));
							}
						} else {
							$this->template->warning(_('The specified user was not found. It is also possible that you may have been removed from the system.'));
						}
					}
				}

				// Load views.
				$view = $this->factory('views');

				// Set Button.
				$view->set('change_password', _('Change Password'));

				// Set Values.
				$view->set('self_url', $this->navigation->selfUrl());
				$view->set('user_name', $edit['user_name']);
				$view->set('password1', $password1);
				$view->set('password2', $password2);
				$view->set('post_validation', $this->security->postValidation());
				$view->set('eun', $eun);

				// Output Template.
				$view->show();
			}
		}
	}
}

return 'NewPassword';
