<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: Handles registration.
 * @author Jason Schoeman
 * @return string
 */
class Register extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Load Extra Classes ////////////////////////////////////////////////////////////////////////////////////////////////////
		$email = $this->factory('mailer'); ///////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Get setting values.
		$settings = $this->db->getSettings(array('allow_registration', 'verify_registration', 'move_verified_group', 'move_verified_role', 'registration_group', 'registration_role', 'email_new_registrations', 'setting_admin_email', 'languages_available', 'registration_message', 'reg_email_direct', 'reg_email_verify', 'reg_email_approve', 'reg_email_admin'));
		// Require JS.
		$this->template->addJsFileToHead('themes/cloud/js/password/jquery.password.js');
		$shortPass = _('Too Short');
		$badPass = _('Weak');
		$goodPass = _('Good');
		$strongPass = _('Strong');
		$samePass = _('Username and Password identical!');
		$passwordMeter = <<<JS
           $(document).ready(function() {
                    $.fn.shortPass = '{$shortPass}';
                    $.fn.badPass = '{$badPass}';
                    $.fn.goodPass = '{$goodPass}';
                    $.fn.strongPass = '{$strongPass}';
                    $.fn.samePassword = '{$samePass}';
                    $.fn.resultStyle = "";
                    $(".password_test").passStrength({
                        shortPass: 		"critical",
                        badPass:		"warning",
                        goodPass:		"notice",
                        strongPass:		"ok",
                        baseStyle:		"passwordMeter",
                        userid:         "#user_name_test"
                    });
            });
JS;
		$this->template->addJsToHead($passwordMeter);
		// Define.
		$edit = array('user_name' => false, 'user_display_name' => false, 'user_email' => false, 'language' => false, 'region' => false, 'user_timezone' => false);
		$approval_url_inset = '';
		$ban_url_inset = '';
		$extra_token_options_html = '';
		$token_key_field = '';
		$extra_language_html = '';
		$extra_region_html = '';
		$timezone_html = '';
		$token_key_field_type = '';
		$token_id = '';
		$registration_selection = '';

		// Switch registration options.
		switch ($settings['allow_registration']) {
			// No registrations accepted.
			case 0:
				// Registration Information.
				$this->template->heading(_('Account Registration Disabled'));
				$this->template->info(_('You cannot register on this system, public registration was disabled by the System Administrator.'));
				// No registration allowed.
				$this->template->warning(_('Please contact the System Administrator requesting to remove the registration link or for him to enable public registrations on this system.'));
				break;
			// All, allow default registrations and token registrations.
			case 1:
				// Registration Information.
				$this->template->heading(_('Register Special or Private Account'));
				$this->template->info(_('You may register a new special or private account, after successful registration you can log-in to use the system.'));
				// Reg info.
				$reg_info = _('Please complete all mandatory fields for successful registration.');
				// Check if a key was used, if so, we do not need selection dropdown.
				if (empty($this->security->get['token_key'])) {
					$registration_selection = $this->db->invokeQuery('PHPDS_SelectTokensQuery');
				}
				// Check if we have a registration token field required.
				// Get registration token field.
				$field_registration_tokens = $this->db->invokeQuery('PHPDS_CountTokensQuery');
				// Check if we should display the token form field.
				if ($field_registration_tokens > 0) {
					// Info for token field.
					$reg_info .= _('If you received a registration token key, please enter this key next to "Registration Token Key", if it was not automatically completed already.');
				}
				// Determine token key field.
				if (!empty($this->security->get['token_key'])) {
					$token_key_field_type = 'class="boxdisabled" readonly';
				} else {
					$token_key_field_type = 'class="boxnormal"';
				}
				// Set unique registration info.
				if (empty($this->security->post)) $this->template->notice($reg_info);
				break;
			// Default registrations only.
			case 2:
				// Registration Information.
				$this->template->heading(_('Register Private Account'));
				$this->template->info(_('You may register a new account, after successful registration you can log-in to use the system.'));
				if (empty($this->security->post))
						$this->template->notice(_('Please complete all mandatory fields for successful registration.'));
				break;
			// Token registrations only, only users with registration tokens can register.
			case 3:
				// Registration Information.
				$this->template->heading(_('Register Special Account'));
				$this->template->info(_('You may register a special or invitation account only, after successful registration you can log-in to use the system.'));
				if (empty($this->security->post))
						$this->template->notice(_('Please complete all mandatory fields for successful registration. You must provide a "Registration Token Key" if it was not automatically inserted.'));
				// Determine token key field.
				if (!empty($this->security->get['token_key'])) {
					$token_key_field_type = 'class="boxdisabled" readonly';
				} else {
					$token_key_field_type = 'class="boxmand"';
				}
				break;
		}
		// Check if we have a registration message to display.
		if (!empty($settings['registration_message']))
				$this->template->message($settings['registration_message']);
		// Set token key field.
		if ($settings['allow_registration'] == 1 || $settings['allow_registration'] == 3) {
			// Set.
			if (!empty($this->security->request['token_key'])) {
				$token_key___ = '<input type="text" size="50" name="token_key" value="' . $this->security->request['token_key'] . '" %s>';
			} else {
				$token_key___ = '<input type="text" size="50" name="token_key" value="" %s>';
			}
			// Create token field form.
			$token_key_field = $token_key___;
		} else {
			$token_key_field = false;
		}
		if ((boolean) $settings['allow_registration'] == true) {
			if (!empty($this->security->post['save'])) {
				// Recieve post variables.
				$edit = $this->security->post;
				$edit['user_name'] = preg_replace('/ /', '', $this->security->post['user_name']);
				$edit['user_password'] = md5($this->security->post['user_password']);
				$edit['verify_password'] = md5($this->security->post['verify_password']);

				// If all fields are completed go to next level.
				if (empty($edit['user_name']) || empty($edit['user_password']) || empty($edit['user_display_name']) || empty($edit['user_email'])) {
					$this->template->warning(_('You have missed some fields, try again!'), false, false);
				} else {
					// Do the matching query to check if any users of this name, email or display name exists.
					$check_user_array = $this->db->invokeQuery('PHPDS_UserDetailQuery', $edit['user_name'], $edit['user_display_name'], $edit['user_email']);
					// Get the results.
					$user_name_ = $check_user_array['user_name'];
					$user_display_name_ = $check_user_array['user_display_name'];
					$user_email_ = $check_user_array['user_email'];
					/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
					/////////////////////// Below is a set of error checkings that the user must be approved by before he is submitted //////////////////////
					/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
					// Check if user already exists.
					if ($user_name_ == $edit['user_name']) {
						$this->template->warning(_('The username you entered already exists, please specify a different username.'), false, false);
						$error[1] = true;
					}
					// Check if the user display name already exists.
					if ($user_display_name_ == $edit['user_display_name']) {
						$this->template->warning(_('The user display name you entered already exists, please specify a different display name.'), false, false);
						$error[2] = true;
					}
					// Check if this email already exists.
					if ($user_email_ == $edit['user_email']) {
						$this->template->warning(_('The user email you entered already exists, are you perhaps already registered, If not please specify a different user email address.'), false, false);
						$error[3] = true;
					}
					// Check email string validity.
					if (!$email->validate($edit['user_email'])) {
						$this->template->warning(_('The email address you specified seems to be invalid. Please verify that your email address is entered correctly.'), false, false);
						$error[4] = true;
					}
					// Check if password is correct length.
					if (strlen($this->security->post['user_password']) < 6) {
						$this->template->warning(_('Your password is too short, make it at least 6 characters!'), false, false);
						$error[5] = true;
					}
					// Check if password compares with verification password.
					if ($edit['user_password'] !== $edit['verify_password']) {
						$this->template->warning(_('The password does not match the second entered password, please verify.'), false, false);
						$error[6] = true;
					}
					// If user tries to send another email in less then 60 seconds, reply with an error.
					(empty($_SESSION['hold1'])) ? $hold1 = false : $hold1 = $_SESSION['hold1'];
					$seconds_ = $this->configuration['time'] - $hold1;
					if ($seconds_ < 60) {
						$this->template->warning(sprintf(_('You can not register another user within 60 seconds, you have waited %s seconds.'), $seconds_), false, false);
						$error[7] = true;
					}
					// Extended error checking.
					// Switch registration types.
					switch ($settings['allow_registration']) {
						// No registrations accepted.
						case 0:
							// Nothing to do.
							break;
						// All, allow default registrations and token registrations.
						case 1:
							// Do we have a token key attempt?
							if (!empty($edit['token_key'])) {
								// We have a token key request, does it exist?
								// Get token id..
								$token_array = $this->db->invokeQuery('PHPDS_CheckTokenQuery', $edit['token_key']);
								// Check if token failed.
								if (empty($token_array['token_id'])) {
									// Failed, yip it did.
									$this->template->warning(_('The token key you entered was incorrect or depleted, please try another again.'));
									$field5 = 'class="boxerror"';
									$error[8] = true;
								} else {
									// Otherwise the key is correct, lets assign it.
									$token_id = $token_array['token_id'];
									$token_user_role_id = $token_array['user_role_id'];
									$token_user_group_id = $token_array['user_group_id'];
								}
							} else if (!empty($edit['token_id_option'])) {
								// We should now check if we have tokens left to use.
								$token_array = $this->db->invokeQuery('PHPDS_CheckTokenByIdQuery', $edit['token_id_option']);
								// Is this token still available. Lets make sure.
								if (empty($token_array['token_id'])) {
									// Oops seems like tokens are finished.
									$this->template->warning(_('The option you have chosen has depleted and not available anymore, please select a new option.'));
									$error[9] = true;
								} else {
									// Ah, it is safe to assign.
									$token_id = $token_array['token_id'];
									$token_user_role_id = $token_array['user_role_id'];
									$token_user_group_id = $token_array['user_group_id'];
								}
							}
							break;
						// Default registrations only.
						case 2:
							$token_id = 0;
							break;
						// Token registrations only, only users with registration tokens can register.
						case 3:
							// We must have a token key attempt?
							if (!empty($edit['token_key'])) {
								// We have a token key request, does it exist?
								// Get token id..
								$token_array = $this->db->invokeQuery('PHPDS_CheckTokenQuery', $edit['token_key']);
								// Check if token failed.
								if (empty($token_array['token_id'])) {
									// Failed, yip it did.
									$this->template->warning(_('The token key you entered was not correct, please try again.'));
									$field5 = 'class="boxerror"';
									$error[10] = true;
								} else {
									// Otherwise the key is correct, lets assign it.
									$token_id = $token_array['token_id'];
									$token_user_role_id = $token_array['user_role_id'];
									$token_user_group_id = $token_array['user_group_id'];
								}
							} else {
								// We have no token key field error, this is wrong.
								$this->template->warning(_('The token key field is mandatory, please enter your key.'));
								$field5 = 'class="boxerror"';
								$error[11] = true;
							}
							break;
					}
					// You can gain data by resetting the mysql pointer for $this->template->global['select_user'] and pulling the data again.
					// Check to see if all fields are set.
					if (empty($error)) {
						// Phase 1 : Create data and save into db.
						switch ($settings['verify_registration']) {
							// Directly submitted.
							case 0:
								// What role should be assigned.
								if (empty($token_id)) {
									// Assign default registration.
									$db_user_role = $settings['move_verified_role'];
									$db_user_group = $settings['move_verified_group'];
								} else {
									// Assign token registration.
									$db_user_role = $token_user_role_id;
									$db_user_group = $token_user_group_id;
									// Deduct token.
									$deduct_token = true;
								}
								break;
							// Needs email verification.
							case 1:
								// Assign default registration.
								$db_user_role = $settings['registration_role'];
								$db_user_group = $settings['registration_group'];
								break;
							// Needs approval.
							case 2:
								// Assign default registration.
								$db_user_role = $settings['registration_role'];
								$db_user_group = $settings['registration_group'];
								break;
						}
						// Set some time variables for database and time submition check.
						$time_now = $this->configuration['time'];
						$_SESSION['hold1'] = $time_now;
						// Finaly save to the database.
						$edit['user_id'] = $this->db->invokeQuery('PHPDS_WriteRegQuery', $edit['user_display_name'], $edit['user_name'], $edit['user_password'], $edit['user_email'], $db_user_group, $db_user_role, $time_now, $edit['language'], $edit['user_timezone'], $edit['region']);

						if (!empty($edit['user_id'])) {
							// Deduct token for direct registration.
							if (isset($deduct_token) && !empty($token_id) && !empty($edit['user_id'])) {
								$this->db->invokeQuery('PHPDS_UpdateTokensQuery', $token_id);
							}
							// Insert registration approval queue.
							// Check if we have a queue.
							if (!empty($edit['user_id']) && !empty($settings['verify_registration'])) {
								// Set in registration queue.
								$this->db->invokeQuery('PHPDS_UpdateRegQueueQuery', $edit['user_id'], $settings['verify_registration'], $token_id);
							}
							// Phase 2 : Send out mail for success.
							switch ($settings['verify_registration']) {
								// Directly submitted.
								case 0:
									// Admin delete url.
									$delete_url = $this->navigation->buildURL(131201277, "du={$edit['user_id']}");
									// Create delete URL
									$delete_url_inset = sprintf(_("\r\n\r\nClick on the url below to DELETE this user,\r\n%s"), $delete_url);
									// Create email message for notification.
									$verification_message = sprintf(_($settings['reg_email_direct']), $edit['user_display_name'], $this->configuration['scripts_name_version'], $this->configuration['absolute_url']);
									$verification_subject = sprintf(_('Registration request approved at %s.'), $this->configuration['scripts_name_version']);
									$verification_to = $edit['user_email'];
									// Send the verification email.
									$email->sendmail("$verification_to", $verification_subject, $verification_message);
									// Display approved message.
									$this->template->ok(sprintf(_('%s, your application for registration was successful, you may now log in.'), $edit['user_display_name']));

									break;
								// Needs email verification.
								case 1:
									// Admin delete url.
									$delete_url = $this->navigation->buildURL(131201277, "du={$edit['user_id']}");
									// Create delete URL
									$delete_url_inset = sprintf(_("\r\n\r\nClick on the url below to DELETE this user,\r\n%s"), $delete_url);
									// Encrypt Username && Password.
									$encrypted_password = urlencode($this->security->encrypt($edit['user_password']));
									$encrypted_username = urlencode($edit['user_name']);
									$registration_url = $this->navigation->buildURL(2143500606, "fa=$encrypted_password&op=$encrypted_username");
									$verification_message = sprintf(_($settings['reg_email_verify']), $edit['user_display_name'], $this->configuration['scripts_name_version'], $registration_url, $this->configuration['absolute_url']);
									$verification_subject = sprintf(_('Registration verification at %s.'), $this->configuration['scripts_name_version']);
									$verification_to = $edit['user_email'];
									// Send the verification email.
									if ($email->sendmail("$verification_to", $verification_subject, $verification_message)) {
										$this->template->ok(sprintf(_('%s, your application for registration was submitted, please verify your account by responding to the verification email that was sent to the specified email address.'), $edit['user_display_name']));
									} else {
										$this->template->warning(_('An error occurred while trying to sent this email. Please notify the Administrator of this problem.'));
										$this->db->invokeQuery('PHPDS_RollbackQuery');
									}

									break;
								// Needs approval.
								case 2:
									// Admin ban user url.
									$ban_url = $this->navigation->buildURL(1210756465, "bu={$edit['user_id']}");
									// Create ban URL
									$ban_url_inset = sprintf(_("\r\n\r\nClick on the url below to BAN this user,\r\n%s"), $ban_url);
									// Admin approval url.
									$approve_url = $this->navigation->buildURL(1210756465, "aue={$edit['user_id']}");
									// Create approval URL
									$approval_url_inset = sprintf(_("\r\n\r\nClick on the url below to APPROVE this user,\r\n%s"), $approve_url);
									// Admin delete url.
									$delete_url = $this->navigation->buildURL(1210756465, "du={$edit['user_id']}");
									// Create delete URL
									$delete_url_inset = sprintf(_("\r\n\r\nClick on the url below to DELETE this user,\r\n%s"), $delete_url);
									// Create email message for notification.
									$verification_message = sprintf(_($settings['reg_email_approve']), $edit['user_display_name'], $this->configuration['scripts_name_version'], $this->configuration['absolute_url']);
									$verification_subject = sprintf(_('Registration request pending at %s.'), $this->configuration['scripts_name_version']);
									$verification_to = $edit['user_email'];
									// Send the verification email.
									$email->sendmail("$verification_to", $verification_subject, $verification_message);
									// Display ok message.
									$this->template->ok(sprintf(_('%s, your application for registration was submitted, an authorized person will now review your submission for approval, please allow some time for approval process to complete.'), $edit['user_display_name']));

									break;
							}
							// Send a email to the system administrator.
							if ((boolean) $settings['email_new_registrations'] == true) {
								// Create email to admin message.
								$message_reg = sprintf(_($settings['reg_email_admin']), $this->configuration['scripts_name_version'], $edit['user_display_name'], $this->core->formatTimeDate($this->configuration['time']), $edit['user_name'], $this->configuration['scripts_name_version'], $approval_url_inset, $ban_url_inset, $delete_url_inset);
								$subject_reg = sprintf(_('New registration received at %s.'), $this->configuration['scripts_name_version']);
								$admin_to = $settings['setting_admin_email'];
								// Send the verification email.
								if ($email->sendmail("$admin_to", $subject_reg, $message_reg)) {
									$this->template->ok(_('The administrator was notified of this registration.'), false, false);
								} else {
									$this->template->warning(_('An error occurred while trying to send this email. Please notify the Administrator of this problem.'));
								}
							}
						}
					}
				}
			}
			// Set IANA language repository class.
			$iana = $this->factory('iana');
			if (empty($edit['language'])) $edit['language'] = false;
			$language_options = $iana->languageOptions($edit['language']);
			if (empty($edit['region'])) $edit['region'] = false;
			$region_options = $iana->regionOptions($edit['region']);
			// Get TimeZone options.
			$timezone = $this->factory('timeZone');
			$timezone_options = $timezone->timezoneOptions($edit['user_timezone']);
			// Check to see if field values are assigned. If not make them mandatory fields.
			if (empty($field1)) $field1 = 'boxmand';
			if (empty($field2)) $field2 = 'boxmand';
			if (empty($field3)) $field3 = 'boxmand';
			if (empty($field4)) $field4 = 'boxmand';
			// Token key field is a bit trickey.
			if (empty($field5)) {
				$token_key_field = sprintf($token_key_field, $token_key_field_type);
			} else {
				$token_key_field = sprintf($token_key_field, $field5);
			}

			// Load views.
			$view = $this->factory('views');

			// Set Array.
			$view->set('e', $edit);

			// Set Button.
			$view->set('submit_registration', _('Submit Registration'));

			// Set Values.
			$view->set('self_url', $this->navigation->selfUrl());
			$view->set('field1', $field1);
			$view->set('field4', $field4);
			$view->set('field2', $field2);
			$view->set('field3', $field3);
			$view->set('registration_selection', $registration_selection);
			$view->set('token_key_field', $token_key_field);
			$view->set('language_options', $language_options);
			$view->set('region_options', $region_options);
			$view->set('timezone_options', $timezone_options);
			$view->set('date_format_show', $this->core->formatTimeDate($this->configuration['time'], 'default', $edit['user_timezone']));
			$view->set('post_validation', $this->security->postValidation());
			// Output Template.
			$view->show();
		}
	}
}

return 'Register';